
// basic variables
var endpoints = new Array();
endpoints["dbpedia"] = "dbpedia.php";
endpoints["aemet"] = "aemet.php";

var countLocationAttempts     = 0;
var countStationAttempts      = 0;
var locationAttempsRetryAfter = 1000;
var locationAttemptsTimeout    = 10;

// global variables for the object currently considered
var lon = 0.0;
var lat = 0.0;
var stationURI = "";

function init(uri){
   queryForLocation(uri);
    showMessage("title", "This box should show the result of the query in the queryLabel function. If it does not update within a few minutes, something probably got wrong with the code - "+uri);
    showMessage("weatherStations", "This box should show the result of the query in the queryWeatherStations function. If it does not update within a few minutes, something probably got wrong with the code - "+uri);
   showMessage("weather", "This box should show the result of the query in the queryWeather function. If it does not update within a few minutes, something probably got wrong with the code - "+uri);
}

function query(uri){
    queryLabel(uri);
    queryWeatherStations(uri);
    queryWeather();
}

//This is the query taken from session 2, which shows one random label and description of the city URI that has been provided. A better query may be used to replace this one, specially if the group already worked on the queries to show descriptions only in English or their favourite language
function queryLabel(uri){
    var sparqler = new SPARQL.Service(endpoints["dbpedia"]);
    var query = sparqler.createQuery();
    squery = 'select distinct ?label ?desc where {'+
	'<'+uri+'> <http://www.w3.org/2000/01/rdf-schema#label> ?label. '+
	'<'+uri+'> <http://www.w3.org/2000/01/rdf-schema#comment> ?desc. '+
	'} LIMIT 1';
    //alert("query = "+squery);
    query.query(squery, {failure: showMessage("title", "queryLabel:: query for label of "+uri+" failed"), success: function(json){showLabel(json);}});
}

//This query looks for weather stations that are close to the point where DBpedia claims that the city is located. It is important to note that the first part of the function aims at ensuring that the latitude and longitude of the city have been already obtained from DBpedia. The weather stations should be ordered by how close they are to that point
function queryWeatherStations(uri){
    if (lon == 0.0) {
	if (countLocationAttempts >= locationAttemptsTimeout){
	    alert("Waited "+((countLocationAttempts*locationAttempsRetryAfter)/1000)+" seconds, and I still don't have a location to check the stations nearby");
	} else {
	    countLocationAttempts++;
	    setTimeout(function(){queryWeatherStations(uri)}, locationAttempsRetryAfter);
	    return;
	}
    }
    var sparqler = new SPARQL.Service(endpoints["aemet"]);
    var query = sparqler.createQuery();
    // put the right query here... 
	squery = 'PREFIX xsd: <http://www.w3.org/2001/XMLSchema#> '+
	'PREFIX aemet: <http://aemet.linkeddata.es/ontology/> '+
	'PREFIX geo: <http://www.w3.org/2003/01/geo/wgs84_pos#> '+
	'SELECT distinct ?station ?lat ?lon '+
        '(((xsd:float(?lat) - '+lat+') * (xsd:float(?lat) - '+lat+')) + ((xsd:float(?lon) - '+lon+') * (xsd:float(?lon) - '+lon+'))) AS ?squareDistance '+
	'WHERE { '+
	'?station a aemet:WeatherStation . '+
	'?station geo:location ?point . '+
	'?point geo:lat ?lat . '+
	'?point geo:long ?lon . '+
	'FILTER (((xsd:float(?lat) - '+lat+') * (xsd:float(?lat) - '+lat+')) + ((xsd:float(?lon) - '+lon+') * (xsd:float(?lon) - '+lon+')) < 0.5) '+
	'} ORDER BY ?squareDistance '
    //alert("query = "+squery);
    query.query(squery, {failure: showMessage("weatherStations", "queryWeatherStations:: query for weather stations of "+lat+","+lon+" failed"), success: function(json){showWeatherStations(json);}});
}

//This query obtains all of the observations that have been obtained by the closest weather station on a specific day that is specified in the query. The first part of the function aims at making sure that the station URI is assigned.
function queryWeather(){
    if (stationURI == "") {
	if (countStationAttempts >= locationAttemptsTimeout){
	    alert("Waited "+((countStationAttempts*locationAttempsRetryAfter)/1000)+" seconds, and I still don't have the URI of the closest station nearby");
	} else {
	    countStationAttempts++;
	    setTimeout(function(){queryWeather(uri)}, locationAttempsRetryAfter);
	    return;
	}
    }
    var sparqler = new SPARQL.Service(endpoints["aemet"]);
    var query = sparqler.createQuery();
    // put the right query here... 
	squery = 'PREFIX xsd: <http://www.w3.org/2001/XMLSchema#> '+
	'PREFIX ssn: <http://purl.oclc.org/NET/ssnx/ssn#> '+
	'PREFIX aemet: <http://aemet.linkeddata.es/ontology/> '+
	'PREFIX w3ctime: <http://www.w3.org/2006/time#> '+
	'PREFIX geo: <http://www.w3.org/2003/01/geo/wgs84_pos#> '+
	'SELECT distinct ?obs ?prop ?value ?dateTime ?h ?min '+
	'WHERE { '+
	'?obs ssn:observedBy <'+stationURI +'> . '+
	'?obs ssn:observedProperty ?prop . '+
	'?obs aemet:valueOfObservedData ?value . '+
	'?obs aemet:observedDataQuality ?q . '+
	'?obs aemet:observedInInterval ?inter . '+
	'?inter w3ctime:hasBeginning ?instant . '+
	'?instant w3ctime:inDateTime ?dateTime . '+
	'?dateTime w3ctime:hour ?h . '+
	'?dateTime w3ctime:minute ?min . '+
	'?dateTime w3ctime:day ?day . '+
	'?dateTime w3ctime:month ?month . '+
	'?dateTime w3ctime:year ?year . '+
	'FILTER (?year = 2011 && ?month = 09 && ?day = 18) '+
	'} ORDER BY ?h ?min'
    //alert("query = "+squery);
    query.query(squery, {failure: showMessage("weather", "queryWeather:: query for weather of "+stationURI+" failed"), success: function(json){showWeather(json);}});
}


///////////////////////////////////////////////

function showLabel(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("title", "No results for this URI or wrong query in queryLabel");
	return;
    } 
    elem = document.getElementById("title");
    elem.innerHTML = "<h1>"+json['results']['bindings'][0]['label']['value']+"</h1>";
    elem.innerHTML += '<div class="desc">'+json['results']['bindings'][0]['desc']['value']+"</div>";
}


function showWeather(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("weather", "No results for this URI or wrong query in queryWeather");
	return;
    } 
    elem = document.getElementById("weather");    
    elem.innerHTML = "<h2>Weather Results</h2>";
    elem.innerHTML = "<h3>"+stationURI+"</h3>";
    for (x in json['results']['bindings']){
    elem.innerHTML += '<div class="desc">'+json['results']['bindings'][x]['prop']['value']+
     " measured "+json['results']['bindings'][x]['value']['value']+
     " at time "+json['results']['bindings'][x]['h']['value']+":"+json['results']['bindings'][x]['min']['value']+"</div>";
    }
}

function showWeatherStations(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("weatherStations", "No results for this URI or wrong query in queryWeatherStations");
	return;
    } 
    elem = document.getElementById("weatherStations");    
    elem.innerHTML = "<h2>Weather Station Results (ordered by proximity)</h2>";
    for (x in json['results']['bindings']){
    elem.innerHTML += '<div class="desc">'+json['results']['bindings'][x]['squareDistance']['value']+" --- "+json['results']['bindings'][x]['station']['value']+": "+json['results']['bindings'][x]['lat']['value']+","+json['results']['bindings'][x]['lon']['value']+"</div>";
  //  elem.innerHTML += '<div class="desc">'+json['results']['bindings'][0]['value']['value']+"</div>";
    }
    stationURI = json['results']['bindings'][0]['station']['value'];
}


// This includes a select query that retrieves the coordinates of the location of the place
function queryForLocation(uri){
    var sparqler = new SPARQL.Service(endpoints["dbpedia"]);
    var query = sparqler.createQuery();
    // put the right query here... careful about selecting the language... 
    squery = 'select distinct ?lat ?lon where {<'+uri+'> <http://www.w3.org/2003/01/geo/wgs84_pos#lat> ?lat. <'+uri+'> <http://www.w3.org/2003/01/geo/wgs84_pos#long> ?lon} limit 1';
        //alert("query = "+squery); 
    query.query(squery, {failure: showMessage("weatherStations", "queryForLocation:: query for location of "+uri+" failed"), 
    success: function(json){recordLocation(json);}});
}

function recordLocation(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("weatherStations", "No results for location of URI or wrong query in queryForLocation");
	return;
    } 
    lat = json['results']['bindings'][0]['lat']['value'];
    lon = json['results']['bindings'][0]['lon']['value'];
    //alert("Lat:"+lat+",lon:"+lon);
}

//This function is avaiable in case that we want to show some RDF facts.
function showBasicFacts(json, uri){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("basic_facts", "No results for this URI or wrong query in queryBasicFacts");
	return;
    } 
    elem = document.getElementById("basic_facts");
    elem.innerHTML = "<h2>Basic Facts</h2>";
    for (x in json['results']['bindings']){
	if (x > 100) break;
	if (json['results']['bindings'][x]['s']['value'] == uri){	    
	    elem.innerHTML+= '<span class="bfact">'+
		"has "+getFragment(json['results']['bindings'][x]['p']['value'])+" "+getFragment(json['results']['bindings'][x]['o']['value'])+
		'</span><br/>';
	} else if (json['results']['bindings'][x]['o']['value'] == uri){	    
	    elem.innerHTML+= '<span class="bfact">'+
		'is '+getFragment(json['results']['bindings'][x]['p']['value'])+" of "+getFragment(json['results']['bindings'][x]['s']['value'])+
		'</span><br/>';
	    
	}
    }
}



